@@ -8,7 +8,7 @@ import functools
 import itertools
 import re
 
-from ._compat import string_types, with_metaclass
+from ._compat import with_metaclass
 from .version import Version, LegacyVersion, parse
 
 
@@ -112,7 +112,7 @@ class _IndividualSpecifier(BaseSpecifier):
         return hash(self._spec)
 
     def __eq__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             try:
                 other = self.__class__(other)
             except InvalidSpecifier:
@@ -123,7 +123,7 @@ class _IndividualSpecifier(BaseSpecifier):
         return self._spec == other._spec
 
     def __ne__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             try:
                 other = self.__class__(other)
             except InvalidSpecifier:
@@ -503,7 +503,7 @@ class Specifier(_IndividualSpecifier):
                 return False
 
         # Ensure that we do not allow a local version of the version mentioned
-        # in the specifier, which is techincally greater than, to match.
+        # in the specifier, which is technically greater than, to match.
         if prospective.local is not None:
             if Version(prospective.base_version) == Version(spec.base_version):
                 return False
@@ -589,7 +589,7 @@ def _pad_version(left, right):
 class SpecifierSet(BaseSpecifier):
 
     def __init__(self, specifiers="", prereleases=None):
-        # Split on , to break each indidivual specifier into it's own item, and
+        # Split on , to break each individual specifier into it's own item, and
         # strip each item to remove leading/trailing whitespace.
         specifiers = [s.strip() for s in specifiers.split(",") if s.strip()]
 
@@ -625,7 +625,7 @@ class SpecifierSet(BaseSpecifier):
         return hash(self._specs)
 
     def __and__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             other = SpecifierSet(other)
         elif not isinstance(other, SpecifierSet):
             return NotImplemented
@@ -648,7 +648,7 @@ class SpecifierSet(BaseSpecifier):
         return specifier
 
     def __eq__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             other = SpecifierSet(other)
         elif isinstance(other, _IndividualSpecifier):
             other = SpecifierSet(str(other))
@@ -658,7 +658,7 @@ class SpecifierSet(BaseSpecifier):
         return self._specs == other._specs
 
     def __ne__(self, other):
-        if isinstance(other, string_types):
+        if isinstance(other, str):
             other = SpecifierSet(other)
         elif isinstance(other, _IndividualSpecifier):
             other = SpecifierSet(str(other))
