@@ -1,7 +1,7 @@
 from matplotlib import pyplot
 from shapely.geometry import LineString
 
-from figures import SIZE
+from figures import SIZE, set_limits, plot_coords, plot_bounds, plot_line_issimple
 
 COLOR = {
     True:  '#6699cc',
@@ -31,16 +31,11 @@ line = LineString([(0, 0), (1, 1), (0, 2), (2, 2), (3, 1), (1, 0)])
 
 plot_coords(ax, line)
 plot_bounds(ax, line)
-plot_line(ax, line)
+plot_line_issimple(ax, line, alpha=0.7)
 
 ax.set_title('a) simple')
 
-xrange = [-1, 4]
-yrange = [-1, 3]
-ax.set_xlim(*xrange)
-ax.set_ylim(*yrange)
-ax.set_yticks(list(range(*yrange)) + [yrange[-1]])
-ax.set_aspect(1)
+set_limits(ax, -1, 4, -1, 3)
 
 #2: complex line
 ax = fig.add_subplot(122)
@@ -48,16 +43,11 @@ line2 = LineString([(0, 0), (1, 1), (0, 2), (2, 2), (-1, 1), (1, 0)])
 
 plot_coords(ax, line2)
 plot_bounds(ax, line2)
-plot_line(ax, line2)
+plot_line_issimple(ax, line2, alpha=0.7)
 
 ax.set_title('b) complex')
 
-xrange = [-2, 3]
-yrange = [-1, 3]
-ax.set_xlim(*xrange)
-ax.set_ylim(*yrange)
-ax.set_yticks(list(range(*yrange)) + [yrange[-1]])
-ax.set_aspect(1)
+set_limits(ax, -2, 3, -1, 3)
 
 pyplot.show()
 
