@@ -14,28 +14,23 @@ import math
 import sys
 from warnings import warn
 from functools import wraps
+import warnings
 
 from shapely.affinity import affine_transform
 from shapely.coords import CoordinateSequence
-from shapely.errors import WKBReadingError, WKTReadingError
+from shapely.errors import GeometryTypeError, WKBReadingError, WKTReadingError
+from shapely.errors import ShapelyDeprecationWarning
 from shapely.geos import WKBWriter, WKTWriter
 from shapely.geos import lgeos
 from shapely.impl import DefaultImplementation, delegated
 
 log = logging.getLogger(__name__)
 
-if sys.version_info[0] < 3:
-    range = xrange
-    integer_types = (int, long)
-else:
-    integer_types = (int,)
-
-
 try:
     import numpy as np
-    integer_types = integer_types + (np.integer,)
+    integer_types = (int, np.integer)
 except ImportError:
-    pass
+    integer_types = (int,)
 
 
 GEOMETRY_TYPES = [
@@ -61,9 +56,9 @@ def dump_coords(geom):
         return geom.exterior.coords[:] + [i.coords[:] for i in geom.interiors]
     elif geom.type.startswith('Multi') or geom.type == 'GeometryCollection':
         # Recursive call
-        return [dump_coords(part) for part in geom]
+        return [dump_coords(part) for part in geom.geoms]
     else:
-        raise ValueError('Unhandled geometry type: ' + repr(geom.type))
+        raise GeometryTypeError('Unhandled geometry type: ' + repr(geom.type))
 
 
 def geometry_type_name(g):
@@ -86,7 +81,7 @@ def geom_factory(g, parent=None):
         [geom_type],
         )
     ob.__class__ = getattr(mod, geom_type)
-    ob._geom = g
+    ob._set_geom(g)
     ob.__p__ = parent
     if lgeos.methods['has_z'](g):
         ob._ndim = 3
@@ -96,26 +91,6 @@ def geom_factory(g, parent=None):
     return ob
 
 
-def geom_from_wkt(data):
-    warn("`geom_from_wkt` is deprecated. Use `geos.wkt_reader.read(data)`.",
-         DeprecationWarning)
-    if sys.version_info[0] >= 3:
-        data = data.encode('ascii')
-    geom = lgeos.GEOSGeomFromWKT(c_char_p(data))
-    if not geom:
-        raise WKTReadingError(
-            "Could not create geometry because of errors while reading input.")
-    return geom_factory(geom)
-
-
-def geom_to_wkt(ob):
-    warn("`geom_to_wkt` is deprecated. Use `geos.wkt_writer.write(ob)`.",
-         DeprecationWarning)
-    if ob is None or ob._geom is None:
-        raise ValueError("Null geometry supports no operations")
-    return lgeos.GEOSGeomToWKT(ob._geom)
-
-
 def deserialize_wkb(data):
     geom = lgeos.GEOSGeomFromWKB_buf(c_char_p(data), c_size_t(len(data)))
     if not geom:
@@ -124,25 +99,10 @@ def deserialize_wkb(data):
     return geom
 
 
-def geom_from_wkb(data):
-    warn("`geom_from_wkb` is deprecated. Use `geos.wkb_reader.read(data)`.",
-         DeprecationWarning)
-    return geom_factory(deserialize_wkb(data))
-
-
-def geom_to_wkb(ob):
-    warn("`geom_to_wkb` is deprecated. Use `geos.wkb_writer.write(ob)`.",
-         DeprecationWarning)
-    if ob is None or ob._geom is None:
-        raise ValueError("Null geometry supports no operations")
-    size = c_size_t()
-    return lgeos.GEOSGeomToWKB_buf(c_void_p(ob._geom), pointer(size))
-
-
 def geos_geom_from_py(ob, create_func=None):
     """Helper function for geos_*_from_py functions in each geom type.
 
-    If a create_func is specified the coodinate sequence is cloned and a new
+    If a create_func is specified the coordinate sequence is cloned and a new
     geometry is created with it, otherwise the geometry is cloned directly.
     This behaviour is useful for converting between LineString and LinearRing
     objects.
@@ -169,13 +129,13 @@ def exceptNull(func):
     return wrapper
 
 
-class CAP_STYLE(object):
+class CAP_STYLE:
     round = 1
     flat = 2
     square = 3
 
 
-class JOIN_STYLE(object):
+class JOIN_STYLE:
     round = 1
     mitre = 2
     bevel = 3
@@ -183,7 +143,7 @@ class JOIN_STYLE(object):
 EMPTY = deserialize_wkb(a2b_hex(b'010700000000000000'))
 
 
-class BaseGeometry(object):
+class BaseGeometry:
     """
     Provides GEOS spatial predicates and topological operations.
 
@@ -222,6 +182,13 @@ class BaseGeometry(object):
     _lgeos = lgeos
 
     def empty(self, val=EMPTY):
+        warn(
+            "The 'empty()' method is deprecated and will be removed in "
+            "Shapely 2.0",
+            ShapelyDeprecationWarning, stacklevel=2)
+        self._empty(val=val)
+
+    def _empty(self, val=EMPTY):
         if not self._other_owned and self.__geom__ and self.__geom__ != EMPTY:
             try:
                 self._lgeos.GEOSGeom_destroy(self.__geom__)
@@ -239,7 +206,7 @@ class BaseGeometry(object):
         return self.__bool__()
 
     def __del__(self):
-        self.empty(val=None)
+        self._empty(val=None)
         self.__p__ = None
 
     def __str__(self):
@@ -250,7 +217,7 @@ class BaseGeometry(object):
         return (self.__class__, (), self.wkb)
 
     def __setstate__(self, state):
-        self.empty()
+        self._empty()
         self.__geom__ = deserialize_wkb(state)
         self._is_empty = False
         if lgeos.methods['has_z'](self.__geom__):
@@ -264,10 +231,35 @@ class BaseGeometry(object):
 
     @_geom.setter
     def _geom(self, val):
-        self.empty()
+        warn(
+            "Setting the '_geom' attribute directly is deprecated, and will "
+            "no longer be possible in Shapely 2.0.",
+            ShapelyDeprecationWarning, stacklevel=2)
+        self._set_geom(val)
+
+    def _set_geom(self, val):
+        self._empty()
         self._is_empty = val in [EMPTY, None]
         self.__geom__ = val
 
+    def __setattr__(self, name, value):
+        # first try regular attribute access via __getattribute__, so that
+        # our own (existing) attributes don't raise a warning
+        try:
+            object.__getattribute__(self, name)
+            super().__setattr__(name, value)
+            return
+        except AttributeError:
+            pass
+
+        # if custom atribute, raise the warning
+        warn(
+            "Setting custom attributes on geometry objects is deprecated, "
+            "and will raise an AttributeError in Shapely 2.0",
+            ShapelyDeprecationWarning, stacklevel=2
+        )
+        super().__setattr__(name, value)
+
     # Operators
     # ---------
 
@@ -298,12 +290,20 @@ class BaseGeometry(object):
     # ---------------------------
 
     @property
-    def ctypes(self):
-        """Return ctypes buffer"""
+    def _ctypes(self):
         raise NotImplementedError
 
     @property
-    def array_interface_base(self):
+    def ctypes(self):
+        """Return ctypes buffer"""
+        warn(
+            "Accessing the 'ctypes' attribute is deprecated,"
+            " and will not be possible any more in Shapely 2.0",
+            ShapelyDeprecationWarning, stacklevel=2)
+        return self._ctypes
+
+    @property
+    def _array_interface_base(self):
         if sys.byteorder == 'little':
             typestr = '<f8'
         elif sys.byteorder == 'big':
@@ -314,22 +314,28 @@ class BaseGeometry(object):
         return {
             'version': 3,
             'typestr': typestr,
-            'data': self.ctypes,
+            'data': self._ctypes,
             }
 
+    @property
+    def array_interface_base(self):
+        warn(
+            "The 'array_interface_base' property is deprecated and will be "
+            "removed in Shapely 2.0.",
+            ShapelyDeprecationWarning, stacklevel=2)
+        return self._array_interface_base()
+
     @property
     def __array_interface__(self):
         """Provide the Numpy array protocol."""
-        raise NotImplementedError
+        raise AttributeError("Not implemented")
 
     # Coordinate access
     # -----------------
 
     def _get_coords(self):
         """Access to geometry's coordinates (CoordinateSequence)"""
-        if self.is_empty:
-            return []
-        return CoordinateSequence(self)
+        raise NotImplementedError("set_coords must be provided by derived classes")
 
     def _set_coords(self, ob):
         raise NotImplementedError(
@@ -359,16 +365,6 @@ class BaseGeometry(object):
     def type(self):
         return self.geometryType()
 
-    def to_wkb(self):
-        warn("`to_wkb` is deprecated. Use the `wkb` property.",
-             DeprecationWarning)
-        return geom_to_wkb(self)
-
-    def to_wkt(self):
-        warn("`to_wkt` is deprecated. Use the `wkt` property.",
-             DeprecationWarning)
-        return geom_to_wkt(self)
-
     @property
     def wkt(self):
         """WKT representation of the geometry"""
@@ -604,9 +600,9 @@ class BaseGeometry(object):
         >>> from shapely.wkt import loads
         >>> g = loads('POINT (0.0 0.0)')
         >>> g.buffer(1.0).area        # 16-gon approx of a unit radius circle
-        3.1365484905459389
+        3.1365484905459384
         >>> g.buffer(1.0, 128).area   # 128-gon approximation
-        3.1415138011443009
+        3.141513801144299
         >>> g.buffer(1.0, 3).area     # triangle approximation
         3.0
         >>> list(g.buffer(1.0, cap_style=CAP_STYLE.square).exterior.coords)
@@ -639,10 +635,6 @@ class BaseGeometry(object):
         if cap_style == CAP_STYLE.round and join_style == JOIN_STYLE.round:
             return geom_factory(self.impl['buffer'](self, distance, res))
 
-        if 'buffer_with_style' not in self.impl:
-            raise NotImplementedError("Styled buffering not available for "
-                                      "GEOS versions < 3.2.")
-
         return geom_factory(self.impl['buffer_with_style'](self, distance, res,
                                                            cap_style,
                                                            join_style,
@@ -664,6 +656,27 @@ class BaseGeometry(object):
             op = self.impl['simplify']
         return geom_factory(op(self, tolerance))
 
+    def normalize(self):
+        """Converts geometry to normal form (or canonical form).
+
+        This method orders the coordinates, rings of a polygon and parts of
+        multi geometries consistently. Typically useful for testing purposes
+        (for example in combination with `equals_exact`).
+
+        Examples
+        --------
+        >>> from shapely.wkt import loads
+        >>> p = loads("MULTILINESTRING((0 0, 1 1), (3 3, 2 2))")
+        >>> p.normalize().wkt
+        'MULTILINESTRING ((2 2, 3 3), (0 0, 1 1))'
+        """
+        # self.impl['normalize'](self)
+        if self._geom is None:
+            raise InvalidGeometryError("Null geometry supports no operations")
+        geom_cloned = lgeos.GEOSGeom_clone(self._geom)
+        lgeos.GEOSNormalize(geom_cloned)
+        return geom_factory(geom_cloned)
+
     # Binary operations
     # -----------------
 
@@ -742,6 +755,10 @@ class BaseGeometry(object):
         """Returns True if the geometry covers the other, else False"""
         return bool(self.impl['covers'](self, other))
 
+    def covered_by(self, other):
+        """Returns True if the geometry is covered by the other, else False"""
+        return bool(self.impl['covered_by'](self, other))
+
     def contains(self, other):
         """Returns True if the geometry contains the other, else False"""
         return bool(self.impl['contains'](self, other))
@@ -755,10 +772,25 @@ class BaseGeometry(object):
         return bool(self.impl['disjoint'](self, other))
 
     def equals(self, other):
-        """Returns True if geometries are equal, else False
+        """Returns True if geometries are equal, else False.
+
+        This method considers point-set equality (or topological
+        equality), and is equivalent to (self.within(other) &
+        self.contains(other)).
+
+        Examples
+        --------
+        >>> LineString(
+        ...     [(0, 0), (2, 2)]
+        ... ).equals(
+        ...     LineString([(0, 0), (1, 1), (2, 2)])
+        ... )
+        True
+
+        Returns
+        -------
+        bool
 
-        Refers to point-set equality (or topological equality), and is equivalent to
-        (self.within(other) & self.contains(other))
         """
         return bool(self.impl['equals'](self, other))
 
@@ -779,22 +811,77 @@ class BaseGeometry(object):
         return bool(self.impl['within'](self, other))
 
     def equals_exact(self, other, tolerance):
-        """Returns True if geometries are equal to within a specified
-        tolerance
+        """True if geometries are equal to within a specified
+        tolerance.
+
+        Parameters
+        ----------
+        other : BaseGeometry
+            The other geometry object in this comparison.
+        tolerance : float
+            Absolute tolerance in the same units as coordinates.
+
+        This method considers coordinate equality, which requires
+        coordinates to be equal and in the same order for all components
+        of a geometry.
+
+        Because of this it is possible for "equals()" to be True for two
+        geometries and "equals_exact()" to be False.
+
+        Examples
+        --------
+        >>> LineString(
+        ...     [(0, 0), (2, 2)]
+        ... ).equals_exact(
+        ...     LineString([(0, 0), (1, 1), (2, 2)]),
+        ...     1e-6
+        ... )
+        False
+
+        Returns
+        -------
+        bool
 
-        Refers to coordinate equality, which requires coordinates to be equal
-        and in the same order for all components of a geometry
         """
         return bool(self.impl['equals_exact'](self, other, tolerance))
 
     def almost_equals(self, other, decimal=6):
-        """Returns True if geometries are equal at all coordinates to a
-        specified decimal place
+        """True if geometries are equal at all coordinates to a
+        specified decimal place.
+
+        .. deprecated:: 1.8.0 The 'almost_equals()' method is deprecated
+            and will be removed in Shapely 2.0 because the name is
+            confusing. The 'equals_exact()' method should be used
+            instead.
+
+        Refers to approximate coordinate equality, which requires
+        coordinates to be approximately equal and in the same order for
+        all components of a geometry.
+
+        Because of this it is possible for "equals()" to be True for two
+        geometries and "almost_equals()" to be False.
+
+        Examples
+        --------
+        >>> LineString(
+        ...     [(0, 0), (2, 2)]
+        ... ).equals_exact(
+        ...     LineString([(0, 0), (1, 1), (2, 2)]),
+        ...     1e-6
+        ... )
+        False
+
+        Returns
+        -------
+        bool
 
-        Refers to approximate coordinate equality, which requires coordinates be
-        approximately equal and in the same order for all components of a geometry.
         """
-        return self.equals_exact(other, 0.5 * 10**(-decimal))
+        warn(
+            "The 'almost_equals()' method is deprecated and will be removed in Shapely 2.0",
+            ShapelyDeprecationWarning,
+            stacklevel=2,
+        )
+        return self.equals_exact(other, 0.5 * 10 ** (-decimal))
 
     def relate_pattern(self, other, pattern):
         """Returns True if the DE-9IM string code for the relationship between
@@ -851,8 +938,8 @@ class BaseMultipartGeometry(BaseGeometry):
     @property
     def __array_interface__(self):
         """Provide the Numpy array protocol."""
-        raise NotImplementedError("Multi-part geometries do not themselves "
-                                  "provide the array interface")
+        raise AttributeError("Multi-part geometries do not themselves "
+                             "provide the array interface")
 
     def _get_coords(self):
         raise NotImplementedError("Sub-geometries may have coordinate "
@@ -877,18 +964,43 @@ class BaseMultipartGeometry(BaseGeometry):
         return self.is_empty is False
 
     def __iter__(self):
+        """
+        .. deprecated:: 1.8
+           Iteration over multi-part geometries is deprecated and will be removed in
+           Shapely 2.0. Use the `geoms` property to access the constituent parts of
+           a multi-part geometry.
+        """
+        warn(
+            "Iteration over multi-part geometries is deprecated and will be removed in "
+            "Shapely 2.0. Use the `geoms` property to access the constituent parts of "
+            "a multi-part geometry.", ShapelyDeprecationWarning, stacklevel=2)
         if not self.is_empty:
             return iter(self.geoms)
         else:
             return iter([])
 
     def __len__(self):
+        warn(
+            "__len__ for multi-part geometries is deprecated and will be removed in "
+            "Shapely 2.0. Check the length of the `geoms` property instead to get the "
+            " number of parts of a multi-part geometry.",
+            ShapelyDeprecationWarning, stacklevel=2)
         if not self.is_empty:
             return len(self.geoms)
         else:
             return 0
 
     def __getitem__(self, index):
+        """
+        .. deprecated:: 1.8
+           __getitem__ for multi-part geometries is deprecated and will be removed in
+           Shapely 2.0. Use the `geoms` property to access the constituent parts of
+           a multi-part geometry.
+        """
+        warn(
+            "__getitem__ for multi-part geometries is deprecated and will be removed in "
+            "Shapely 2.0. Use the `geoms` property to access the constituent parts of "
+            "a multi-part geometry.", ShapelyDeprecationWarning, stacklevel=2)
         if not self.is_empty:
             return self.geoms[index]
         else:
@@ -897,8 +1009,8 @@ class BaseMultipartGeometry(BaseGeometry):
     def __eq__(self, other):
         return (
             type(other) == type(self) and
-            len(self) == len(other) and
-            all(x == y for x, y in zip(self, other))
+            len(self.geoms) == len(other.geoms) and
+            all(x == y for x, y in zip(self.geoms, other.geoms))
         )
 
     def __ne__(self, other):
@@ -922,11 +1034,11 @@ class BaseMultipartGeometry(BaseGeometry):
         if color is None:
             color = "#66cc99" if self.is_valid else "#ff3333"
         return '<g>' + \
-            ''.join(p.svg(scale_factor, color) for p in self) + \
+            ''.join(p.svg(scale_factor, color) for p in self.geoms) + \
             '</g>'
 
 
-class GeometrySequence(object):
+class GeometrySequence:
     """
     Iterative access to members of a homogeneous multipart geometry.
     """
@@ -957,7 +1069,7 @@ class GeometrySequence(object):
     def _get_geom_item(self, i):
         g = self.shape_factory()
         g._other_owned = True
-        g._geom = lgeos.GEOSGetGeometryN(self._geom, i)
+        g._set_geom(lgeos.GEOSGetGeometryN(self._geom, i))
         g._ndim = self._ndim
         g.__p__ = self
         return g
@@ -984,7 +1096,7 @@ class GeometrySequence(object):
             return self._get_geom_item(i)
         elif isinstance(key, slice):
             if type(self) == HeterogeneousGeometrySequence:
-                raise TypeError(
+                raise GeometryTypeError(
                     "Heterogenous geometry collections are not sliceable")
             res = []
             start, stop, stride = key.indices(m)
@@ -1009,7 +1121,7 @@ class HeterogeneousGeometrySequence(GeometrySequence):
     """
 
     def __init__(self, parent):
-        super(HeterogeneousGeometrySequence, self).__init__(parent, None)
+        super().__init__(parent, None)
 
     def _get_geom_item(self, i):
         sub = lgeos.GEOSGetGeometryN(self._geom, i)
@@ -1022,12 +1134,3 @@ class EmptyGeometry(BaseGeometry):
     def __init__(self):
         """Create an empty geometry."""
         BaseGeometry.__init__(self)
-
-
-def _test():
-    """Test runner"""
-    import doctest
-    doctest.testmod()
-
-if __name__ == "__main__":
-    _test()
