@@ -1,13 +1,10 @@
 """Collections of polygons and related utilities
 """
 
-import sys
-
-if sys.version_info[0] < 3:
-    range = xrange
-
 from ctypes import c_void_p, cast
+import warnings
 
+from shapely.errors import ShapelyDeprecationWarning
 from shapely.geos import lgeos
 from shapely.geometry.base import BaseMultipartGeometry, geos_geom_from_py
 from shapely.geometry import polygon
@@ -19,10 +16,10 @@ __all__ = ['MultiPolygon', 'asMultiPolygon']
 class MultiPolygon(BaseMultipartGeometry):
 
     """A collection of one or more polygons
-    
+
     If component polygons overlap the collection is `invalid` and some
     operations on it may fail.
-    
+
     Attributes
     ----------
     geoms : sequence
@@ -42,9 +39,10 @@ class MultiPolygon(BaseMultipartGeometry):
         -------
         Construct a collection from a sequence of coordinate tuples
 
+          >>> from shapely.geometry import Polygon
           >>> ob = MultiPolygon( [
           ...     (
-          ...     ((0.0, 0.0), (0.0, 1.0), (1.0, 1.0), (1.0, 0.0)), 
+          ...     ((0.0, 0.0), (0.0, 1.0), (1.0, 1.0), (1.0, 0.0)),
           ...     [((0.1,0.1), (0.1,0.2), (0.2,0.2), (0.2,0.1))]
           ...     )
           ... ] )
@@ -53,15 +51,17 @@ class MultiPolygon(BaseMultipartGeometry):
           >>> type(ob.geoms[0]) == Polygon
           True
         """
-        super(MultiPolygon, self).__init__()
+        super().__init__()
 
         if not polygons:
             # allow creation of empty multipolygons, to support unpickling
-            pass
+            return
         elif context_type == 'polygons':
-            self._geom, self._ndim = geos_multipolygon_from_polygons(polygons)
+            geom, n = geos_multipolygon_from_polygons(polygons)
         elif context_type == 'geojson':
-            self._geom, self._ndim = geos_multipolygon_from_py(polygons)
+            geom, n = geos_multipolygon_from_py(polygons)
+        self._set_geom(geom)
+        self._ndim = n
 
     def shape_factory(self, *args):
         return polygon.Polygon(*args)
@@ -80,7 +80,7 @@ class MultiPolygon(BaseMultipartGeometry):
             'coordinates': allcoords
             }
 
-    def svg(self, scale_factor=1., fill_color=None):
+    def svg(self, scale_factor=1., fill_color=None, opacity=None):
         """Returns group of SVG path elements for the MultiPolygon geometry.
 
         Parameters
@@ -90,22 +90,30 @@ class MultiPolygon(BaseMultipartGeometry):
         fill_color : str, optional
             Hex string for fill color. Default is to use "#66cc99" if
             geometry is valid, and "#ff3333" if invalid.
+        opacity : float
+            Float number between 0 and 1 for color opacity. Defaul value is 0.6
         """
         if self.is_empty:
             return '<g />'
         if fill_color is None:
             fill_color = "#66cc99" if self.is_valid else "#ff3333"
         return '<g>' + \
-            ''.join(p.svg(scale_factor, fill_color) for p in self) + \
+            ''.join(p.svg(scale_factor, fill_color, opacity) for p in self.geoms) + \
             '</g>'
 
 
 class MultiPolygonAdapter(CachingGeometryProxy, MultiPolygon):
-    
+
     context = None
     _other_owned = False
 
     def __init__(self, context, context_type='polygons'):
+        warnings.warn(
+            "The proxy geometries (through the 'asShape()', 'asMultiPolygon()' "
+            "or 'MultiPolygonAdapter()' constructors) are deprecated and will be "
+            "removed in Shapely 2.0. Use the 'shape()' function or the "
+            "standard 'MultiPolygon()' constructor instead.",
+            ShapelyDeprecationWarning, stacklevel=3)
         self.context = context
         if context_type == 'geojson':
             self.factory = geos_multipolygon_from_py
@@ -134,7 +142,7 @@ def geos_multipolygon_from_py(ob):
     """ob must provide Python geo interface coordinates."""
     L = len(ob)
     assert L >= 1
-    
+
     N = len(ob[0][0][0])
     assert N == 2 or N == 3
 
@@ -181,7 +189,7 @@ def geos_multipolygon_from_polygons(arg):
         N = len(exemplar[0][0])
     except TypeError:
         N = exemplar._ndim
-    
+
     assert N == 2 or N == 3
 
     subs = (c_void_p * L)()
@@ -198,11 +206,3 @@ def geos_multipolygon_from_polygons(arg):
         subs[i] = cast(geom, c_void_p)
 
     return (lgeos.GEOSGeom_createCollection(6, subs, L), N)
-
-# Test runner
-def _test():
-    import doctest
-    doctest.testmod()
-
-if __name__ == "__main__":
-    _test()
