@@ -2,8 +2,10 @@
 """
 
 from ctypes import c_double
+import warnings
 
-from shapely.errors import DimensionError
+from shapely.coords import CoordinateSequence
+from shapely.errors import DimensionError, ShapelyDeprecationWarning
 from shapely.geos import lgeos
 from shapely.geometry.base import BaseGeometry, geos_geom_from_py
 from shapely.geometry.proxy import CachingGeometryProxy
@@ -26,7 +28,7 @@ class Point(BaseGeometry):
     -------
       >>> p = Point(1.0, -1.0)
       >>> print(p)
-      POINT (1.0000000000000000 -1.0000000000000000)
+      POINT (1 -1)
       >>> p.y
       -1.0
       >>> p.x
@@ -45,7 +47,16 @@ class Point(BaseGeometry):
         """
         BaseGeometry.__init__(self)
         if len(args) > 0:
-            self._set_coords(*args)
+            if len(args) == 1:
+                geom, n = geos_point_from_py(args[0])
+            elif len(args) > 3:
+                raise TypeError(
+                    "Point() takes at most 3 arguments ({} given)".format(len(args))
+                )
+            else:
+                geom, n = geos_point_from_py(tuple(args))
+            self._set_geom(geom)
+            self._ndim = n
 
     # Coordinate getters and setters
 
@@ -73,7 +84,7 @@ class Point(BaseGeometry):
             'coordinates': self.coords[0]
             }
 
-    def svg(self, scale_factor=1., fill_color=None):
+    def svg(self, scale_factor=1., fill_color=None, opacity=None):
         """Returns SVG circle element for the Point geometry.
 
         Parameters
@@ -83,18 +94,22 @@ class Point(BaseGeometry):
         fill_color : str, optional
             Hex string for fill color. Default is to use "#66cc99" if
             geometry is valid, and "#ff3333" if invalid.
+        opacity : float
+            Float number between 0 and 1 for color opacity. Defaul value is 0.6
         """
         if self.is_empty:
             return '<g />'
         if fill_color is None:
             fill_color = "#66cc99" if self.is_valid else "#ff3333"
+        if opacity is None:
+            opacity = 0.6 
         return (
             '<circle cx="{0.x}" cy="{0.y}" r="{1}" '
-            'stroke="#555555" stroke-width="{2}" fill="{3}" opacity="0.6" />'
-            ).format(self, 3. * scale_factor, 1. * scale_factor, fill_color)
+            'stroke="#555555" stroke-width="{2}" fill="{3}" opacity="{4}" />'
+            ).format(self, 3. * scale_factor, 1. * scale_factor, fill_color, opacity)
 
     @property
-    def ctypes(self):
+    def _ctypes(self):
         if not self._ctypes_data:
             array_type = c_double * self._ndim
             array = array_type()
@@ -106,15 +121,30 @@ class Point(BaseGeometry):
             self._ctypes_data = array
         return self._ctypes_data
 
-    def array_interface(self):
+    def _array_interface(self):
         """Provide the Numpy array protocol."""
         if self.is_empty:
             ai = {'version': 3, 'typestr': '<f8', 'shape': (0,), 'data': (c_double * 0)()}
         else:
-            ai = self.array_interface_base
+            ai = self._array_interface_base
             ai.update({'shape': (self._ndim,)})
         return ai
-    __array_interface__ = property(array_interface)
+
+    def array_interface(self):
+        """Provide the Numpy array protocol."""
+        warnings.warn(
+            "The 'array_interface' method is deprecated and will be removed "
+            "in Shapely 2.0.",
+            ShapelyDeprecationWarning, stacklevel=2)
+        return self._array_interface()
+
+    @property
+    def __array_interface__(self):
+        warnings.warn(
+            "The array interface is deprecated and will no longer work in "
+            "Shapely 2.0. Convert the '.coords' to a numpy array instead.",
+            ShapelyDeprecationWarning, stacklevel=3)
+        return self._array_interface()
 
     @property
     def bounds(self):
@@ -127,16 +157,26 @@ class Point(BaseGeometry):
 
     # Coordinate access
 
+    def _get_coords(self):
+        """Access to geometry's coordinates (CoordinateSequence)"""
+        return CoordinateSequence(self)
+
     def _set_coords(self, *args):
-        self.empty()
+        warnings.warn(
+            "Setting the 'coords' to mutate a Geometry in place is deprecated,"
+            " and will not be possible any more in Shapely 2.0",
+            ShapelyDeprecationWarning, stacklevel=2)
+        self._empty()
         if len(args) == 1:
-            self._geom, self._ndim = geos_point_from_py(args[0])
+            geom, n = geos_point_from_py(args[0])
         elif len(args) > 3:
             raise TypeError("Point() takes at most 3 arguments ({} given)".format(len(args)))
         else:
-            self._geom, self._ndim = geos_point_from_py(tuple(args))
+            geom, n = geos_point_from_py(tuple(args))
+        self._set_geom(geom)
+        self._ndim = n
 
-    coords = property(BaseGeometry._get_coords, _set_coords)
+    coords = property(_get_coords, _set_coords)
 
     @property
     def xy(self):
@@ -157,6 +197,12 @@ class PointAdapter(CachingGeometryProxy, Point):
     _other_owned = False
 
     def __init__(self, context):
+        warnings.warn(
+            "The proxy geometries (through the 'asShape()', 'asPoint()' or "
+            "'PointAdapter()' constructors) are deprecated and will be "
+            "removed in Shapely 2.0. Use the 'shape()' function or the "
+            "standard 'Point()' constructor instead.",
+            ShapelyDeprecationWarning, stacklevel=4)
         self.context = context
         self.factory = geos_point_from_py
 
@@ -180,7 +226,9 @@ class PointAdapter(CachingGeometryProxy, Point):
         except AttributeError:
             return self.array_interface()
 
-    _get_coords = BaseGeometry._get_coords
+    def _get_coords(self):
+        """Access to geometry's coordinates (CoordinateSequence)"""
+        return CoordinateSequence(self)
 
     def _set_coords(self, ob):
         raise NotImplementedError("Adapters can not modify their sources")
@@ -203,7 +251,7 @@ def geos_point_from_py(ob, update_geom=None, update_ndim=0):
         return geos_geom_from_py(ob)
 
     # Accept either (x, y) or [(x, y)]
-    if not hasattr(ob, '__getitem__'):  # Iterators, e.g. Python 3 zip
+    if not hasattr(ob, '__getitem__'):  # generators
         ob = list(ob)
 
     if isinstance(ob[0], tuple):
