@@ -2,20 +2,20 @@
 """
 
 import sys
+import warnings
 
-if sys.version_info[0] < 3:
-    range = xrange
-
-from ctypes import c_double, c_void_p, cast, POINTER
-from ctypes import ArgumentError
+from ctypes import c_void_p, cast, POINTER
 import weakref
 
 from shapely.algorithms.cga import signed_area
-from shapely.coords import required
+from shapely.coords import CoordinateSequence
 from shapely.geos import lgeos
 from shapely.geometry.base import BaseGeometry, geos_geom_from_py
 from shapely.geometry.linestring import LineString, LineStringAdapter
+from shapely.geometry.point import Point
 from shapely.geometry.proxy import PolygonProxy
+from shapely.errors import TopologicalError, ShapelyDeprecationWarning
+
 
 __all__ = ['Polygon', 'asPolygon', 'LinearRing', 'asLinearRing']
 
@@ -33,7 +33,8 @@ class LinearRing(LineString):
         Parameters
         ----------
         coordinates : sequence
-            A sequence of (x, y [,z]) numeric coordinate pairs or triples
+            A sequence of (x, y [,z]) numeric coordinate pairs or triples.
+            Also can be a sequence of Point objects.
 
         Rings are implicitly closed. There is no need to specific a final
         coordinate pair identical to the first.
@@ -50,7 +51,11 @@ class LinearRing(LineString):
         """
         BaseGeometry.__init__(self)
         if coordinates is not None:
-            self._set_coords(coordinates)
+            ret = geos_linearring_from_py(coordinates)
+            if ret is not None:
+                geom, n = ret
+                self._set_geom(geom)
+                self._ndim = n
 
     @property
     def __geo_interface__(self):
@@ -61,14 +66,33 @@ class LinearRing(LineString):
 
     # Coordinate access
 
-    _get_coords = BaseGeometry._get_coords
+    def _get_coords(self):
+        """Access to geometry's coordinates (CoordinateSequence)"""
+        return CoordinateSequence(self)
 
     def _set_coords(self, coordinates):
-        self.empty()
-        self._geom, self._ndim = geos_linearring_from_py(coordinates)
+        warnings.warn(
+            "Setting the 'coords' to mutate a Geometry in place is deprecated,"
+            " and will not be possible any more in Shapely 2.0",
+            ShapelyDeprecationWarning, stacklevel=2)
+        self._empty()
+        ret = geos_linearring_from_py(coordinates)
+        if ret is not None:
+            geom, n = ret
+            self._set_geom(geom)
+            self._ndim = n
 
     coords = property(_get_coords, _set_coords)
 
+    def __setstate__(self, state):
+        """WKB doesn't differentiate between LineString and LinearRing so we
+        need to move the coordinate sequence into the correct geometry type"""
+        super().__setstate__(state)
+        cs = lgeos.GEOSGeom_getCoordSeq(self.__geom__)
+        cs_clone = lgeos.GEOSCoordSeq_clone(cs)
+        lgeos.GEOSGeom_destroy(self.__geom__)
+        self.__geom__ = lgeos.GEOSGeom_createLinearRing(cs_clone)
+
     @property
     def is_ccw(self):
         """True is the ring is oriented counter clock-wise"""
@@ -86,6 +110,12 @@ class LinearRingAdapter(LineStringAdapter):
     __p__ = None
 
     def __init__(self, context):
+        warnings.warn(
+            "The proxy geometries (through the 'asShape()', 'asLinearRing()' or "
+            "'LinearRingAdapter()' constructors) are deprecated and will be "
+            "removed in Shapely 2.0. Use the 'shape()' function or the "
+            "standard 'LinearRing()' constructor instead.",
+            ShapelyDeprecationWarning, stacklevel=3)
         self.context = context
         self.factory = geos_linearring_from_py
 
@@ -96,7 +126,11 @@ class LinearRingAdapter(LineStringAdapter):
             'coordinates': tuple(self.coords)
             }
 
-    coords = property(BaseGeometry._get_coords)
+    def _get_coords(self):
+        """Access to geometry's coordinates (CoordinateSequence)"""
+        return CoordinateSequence(self)
+
+    coords = property(_get_coords)
 
 
 def asLinearRing(context):
@@ -104,7 +138,7 @@ def asLinearRing(context):
     return LinearRingAdapter(context)
 
 
-class InteriorRingSequence(object):
+class InteriorRingSequence:
 
     _factory = None
     _geom = None
@@ -133,9 +167,6 @@ class InteriorRingSequence(object):
         else:
             raise StopIteration
 
-    if sys.version_info[0] < 3:
-        next = __next__
-
     def __len__(self):
         return lgeos.GEOSGetNumInteriorRings(self._geom)
 
@@ -176,7 +207,7 @@ class InteriorRingSequence(object):
         if i not in self.__rings__:
             g = lgeos.GEOSGetInteriorRingN(self._geom, i)
             ring = LinearRing()
-            ring._geom = g
+            ring._set_geom(g)
             ring.__p__ = self
             ring._other_owned = True
             ring._ndim = self._ndim
@@ -209,7 +240,8 @@ class Polygon(BaseGeometry):
         Parameters
         ----------
         shell : sequence
-            A sequence of (x, y [,z]) numeric coordinate pairs or triples
+            A sequence of (x, y [,z]) numeric coordinate pairs or triples.
+            Also can be a sequence of Point objects.
         holes : sequence
             A sequence of objects which satisfy the same requirements as the
             shell parameters above
@@ -226,16 +258,22 @@ class Polygon(BaseGeometry):
         BaseGeometry.__init__(self)
 
         if shell is not None:
-            self._geom, self._ndim = geos_polygon_from_py(shell, holes)
+            ret = geos_polygon_from_py(shell, holes)
+            if ret is not None:
+                geom, n = ret
+                self._set_geom(geom)
+                self._ndim = n
+            else:
+                self._empty()
 
     @property
     def exterior(self):
         if self.is_empty:
-            return None
+            return LinearRing()
         elif self._exterior is None or self._exterior() is None:
             g = lgeos.GEOSGetExteriorRing(self._geom)
             ring = LinearRing()
-            ring._geom = g
+            ring._set_geom(g)
             ring.__p__ = self
             ring._other_owned = True
             ring._ndim = self._ndim
@@ -251,6 +289,11 @@ class Polygon(BaseGeometry):
     def __eq__(self, other):
         if not isinstance(other, Polygon):
             return False
+        check_empty = (self.is_empty, other.is_empty)
+        if all(check_empty):
+            return True
+        elif any(check_empty):
+            return False
         my_coords = [
             tuple(self.exterior.coords),
             [tuple(interior.coords) for interior in self.interiors]
@@ -267,14 +310,14 @@ class Polygon(BaseGeometry):
     __hash__ = None
 
     @property
-    def ctypes(self):
+    def _ctypes(self):
         if not self._ctypes_data:
-            self._ctypes_data = self.exterior.ctypes
+            self._ctypes_data = self.exterior._ctypes
         return self._ctypes_data
 
     @property
     def __array_interface__(self):
-        raise NotImplementedError(
+        raise AttributeError(
         "A polygon does not itself provide the array interface. Its rings do.")
 
     def _get_coords(self):
@@ -292,15 +335,17 @@ class Polygon(BaseGeometry):
 
     @property
     def __geo_interface__(self):
-        coords = [tuple(self.exterior.coords)]
-        for hole in self.interiors:
-            coords.append(tuple(hole.coords))
+        if self.exterior == LinearRing():
+            coords = []
+        else:
+            coords = [tuple(self.exterior.coords)]
+            for hole in self.interiors:
+                coords.append(tuple(hole.coords))
         return {
             'type': 'Polygon',
-            'coordinates': tuple(coords)
-            }
+            'coordinates': tuple(coords)}
 
-    def svg(self, scale_factor=1., fill_color=None):
+    def svg(self, scale_factor=1., fill_color=None, opacity=None):
         """Returns SVG path element for the Polygon geometry.
 
         Parameters
@@ -310,28 +355,47 @@ class Polygon(BaseGeometry):
         fill_color : str, optional
             Hex string for fill color. Default is to use "#66cc99" if
             geometry is valid, and "#ff3333" if invalid.
+        opacity : float
+            Float number between 0 and 1 for color opacity. Defaul value is 0.6
         """
         if self.is_empty:
             return '<g />'
         if fill_color is None:
             fill_color = "#66cc99" if self.is_valid else "#ff3333"
+        if opacity is None:
+            opacity = 0.6 
         exterior_coords = [
-            ["{0},{1}".format(*c) for c in self.exterior.coords]]
+            ["{},{}".format(*c) for c in self.exterior.coords]]
         interior_coords = [
-            ["{0},{1}".format(*c) for c in interior.coords]
+            ["{},{}".format(*c) for c in interior.coords]
             for interior in self.interiors]
         path = " ".join([
-            "M {0} L {1} z".format(coords[0], " L ".join(coords[1:]))
+            "M {} L {} z".format(coords[0], " L ".join(coords[1:]))
             for coords in exterior_coords + interior_coords])
         return (
             '<path fill-rule="evenodd" fill="{2}" stroke="#555555" '
-            'stroke-width="{0}" opacity="0.6" d="{1}" />'
-            ).format(2. * scale_factor, path, fill_color)
+            'stroke-width="{0}" opacity="{3}" d="{1}" />'
+            ).format(2. * scale_factor, path, fill_color, opacity)
+
+    @classmethod
+    def from_bounds(cls, xmin, ymin, xmax, ymax):
+        """Construct a `Polygon()` from spatial bounds."""
+        return cls([
+            (xmin, ymin),
+            (xmin, ymax),
+            (xmax, ymax),
+            (xmax, ymin)])
 
 
 class PolygonAdapter(PolygonProxy, Polygon):
 
     def __init__(self, shell, holes=None):
+        warnings.warn(
+            "The proxy geometries (through the 'asShape()', 'asPolygon()' or "
+            "'PolygonAdapter()' constructors) are deprecated and will be "
+            "removed in Shapely 2.0. Use the 'shape()' function or the "
+            "standard 'Polygon()' constructor instead.",
+            ShapelyDeprecationWarning, stacklevel=4)
         self.shell = shell
         self.holes = holes
         self.context = (shell, holes)
@@ -354,6 +418,7 @@ def asPolygon(shell, holes=None):
     """Adapt objects to the Polygon interface"""
     return PolygonAdapter(shell, holes)
 
+
 def orient(polygon, sign=1.0):
     s = float(sign)
     rings = []
@@ -369,138 +434,113 @@ def orient(polygon, sign=1.0):
             rings.append(list(ring.coords)[::-1])
     return Polygon(rings[0], rings[1:])
 
+
 def geos_linearring_from_py(ob, update_geom=None, update_ndim=0):
     # If a LinearRing is passed in, clone it and return
-    # If a LineString is passed in, clone the coord seq and return a LinearRing
+    # If a valid LineString is passed in, clone the coord seq and return a
+    # LinearRing.
+    #
+    # NB: access to coordinates using the array protocol has been moved
+    # entirely to the speedups module.
+
     if isinstance(ob, LineString):
         if type(ob) == LinearRing:
             return geos_geom_from_py(ob)
+        elif not ob.is_valid:
+            raise TopologicalError("An input LineString must be valid.")
+        elif ob.is_closed and len(ob.coords) >= 4:
+            return geos_geom_from_py(ob, lgeos.GEOSGeom_createLinearRing)
         else:
-            if ob.is_closed and len(ob.coords) >= 4:
-                return geos_geom_from_py(ob, lgeos.GEOSGeom_createLinearRing)
+            ob = list(ob.coords)
 
-    # If numpy is present, we use numpy.require to ensure that we have a
-    # C-continguous array that owns its data. View data will be copied.
-    ob = required(ob)
     try:
-        # From array protocol
-        array = ob.__array_interface__
-        assert len(array['shape']) == 2
-        m = array['shape'][0]
-        n = array['shape'][1]
-        if m < 3:
+        m = len(ob)
+    except TypeError:  # generators
+        ob = list(ob)
+        m = len(ob)
+
+    if m == 0:
+        return None
+
+    def _coords(o):
+        if isinstance(o, Point):
+            return o.coords[0]
+        else:
+            return o
+
+    n = len(_coords(ob[0]))
+    if m < 3:
+        raise ValueError(
+            "A LinearRing must have at least 3 coordinate tuples")
+    assert (n == 2 or n == 3)
+
+    # Add closing coordinates if not provided
+    if (
+        m == 3
+        or _coords(ob[0])[0] != _coords(ob[-1])[0]
+        or _coords(ob[0])[1] != _coords(ob[-1])[1]
+    ):
+        M = m + 1
+    else:
+        M = m
+
+    # Create a coordinate sequence
+    if update_geom is not None:
+        if n != update_ndim:
             raise ValueError(
-                "A LinearRing must have at least 3 coordinate tuples")
-        assert n == 2 or n == 3
+                "Coordinate dimensions mismatch: target geom has {} dims, "
+                "update geom has {} dims".format(n, update_ndim))
+        cs = lgeos.GEOSGeom_getCoordSeq(update_geom)
+    else:
+        cs = lgeos.GEOSCoordSeq_create(M, n)
 
-        # Make pointer to the coordinate array
-        if isinstance(array['data'], tuple):
-            # numpy tuple (addr, read-only)
-            cp = cast(array['data'][0], POINTER(c_double))
-        else:
-            cp = array['data']
+    # add to coordinate sequence
+    for i in range(m):
+        coords = _coords(ob[i])
+        # Because of a bug in the GEOS C API,
+        # always set X before Y
+        lgeos.GEOSCoordSeq_setX(cs, i, coords[0])
+        lgeos.GEOSCoordSeq_setY(cs, i, coords[1])
+        if n == 3:
+            try:
+                lgeos.GEOSCoordSeq_setZ(cs, i, coords[2])
+            except IndexError:
+                raise ValueError("Inconsistent coordinate dimensionality")
 
-        # Add closing coordinates to sequence?
-        if cp[0] != cp[m*n-n] or cp[1] != cp[m*n-n+1]:
-            M = m + 1
-        else:
-            M = m
-
-        # Create a coordinate sequence
-        if update_geom is not None:
-            cs = lgeos.GEOSGeom_getCoordSeq(update_geom)
-            if n != update_ndim:
-                raise ValueError(
-                "Wrong coordinate dimensions; this geometry has dimensions: %d" \
-                % update_ndim)
-        else:
-            cs = lgeos.GEOSCoordSeq_create(M, n)
-
-        # add to coordinate sequence
-        for i in range(m):
-            # Because of a bug in the GEOS C API,
-            # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, i, cp[n*i])
-            lgeos.GEOSCoordSeq_setY(cs, i, cp[n*i+1])
-            if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, i, cp[n*i+2])
-
-        # Add closing coordinates to sequence?
-        if M > m:
-            # Because of a bug in the GEOS C API,
-            # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, M-1, cp[0])
-            lgeos.GEOSCoordSeq_setY(cs, M-1, cp[1])
-            if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, M-1, cp[2])
-
-    except AttributeError:
-        # Fall back on list
-        try:
-            m = len(ob)
-        except TypeError:  # Iterators, e.g. Python 3 zip
-            ob = list(ob)
-            m = len(ob)
-
-        n = len(ob[0])
-        if m < 3:
-            raise ValueError(
-                "A LinearRing must have at least 3 coordinate tuples")
-        assert (n == 2 or n == 3)
-
-        # Add closing coordinates if not provided
-        if m == 3 or ob[0][0] != ob[-1][0] or ob[0][1] != ob[-1][1]:
-            M = m + 1
-        else:
-            M = m
-
-        # Create a coordinate sequence
-        if update_geom is not None:
-            cs = lgeos.GEOSGeom_getCoordSeq(update_geom)
-            if n != update_ndim:
-                raise ValueError(
-                "Wrong coordinate dimensions; this geometry has dimensions: %d" \
-                % update_ndim)
-        else:
-            cs = lgeos.GEOSCoordSeq_create(M, n)
-
-        # add to coordinate sequence
-        for i in range(m):
-            coords = ob[i]
-            # Because of a bug in the GEOS C API,
-            # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, i, coords[0])
-            lgeos.GEOSCoordSeq_setY(cs, i, coords[1])
-            if n == 3:
-                try:
-                    lgeos.GEOSCoordSeq_setZ(cs, i, coords[2])
-                except IndexError:
-                    raise ValueError("Inconsistent coordinate dimensionality")
-
-        # Add closing coordinates to sequence?
-        if M > m:
-            coords = ob[0]
-            # Because of a bug in the GEOS C API,
-            # always set X before Y
-            lgeos.GEOSCoordSeq_setX(cs, M-1, coords[0])
-            lgeos.GEOSCoordSeq_setY(cs, M-1, coords[1])
-            if n == 3:
-                lgeos.GEOSCoordSeq_setZ(cs, M-1, coords[2])
+    # Add closing coordinates to sequence?
+    if M > m:
+        coords = _coords(ob[0])
+        # Because of a bug in the GEOS C API,
+        # always set X before Y
+        lgeos.GEOSCoordSeq_setX(cs, M-1, coords[0])
+        lgeos.GEOSCoordSeq_setY(cs, M-1, coords[1])
+        if n == 3:
+            lgeos.GEOSCoordSeq_setZ(cs, M-1, coords[2])
 
     if update_geom is not None:
         return None
     else:
         return lgeos.GEOSGeom_createLinearRing(cs), n
 
+
 def update_linearring_from_py(geom, ob):
     geos_linearring_from_py(ob, geom._geom, geom._ndim)
 
+
 def geos_polygon_from_py(shell, holes=None):
+
+    if shell is None:
+        return None
+
     if isinstance(shell, Polygon):
         return geos_geom_from_py(shell)
 
     if shell is not None:
-        geos_shell, ndim = geos_linearring_from_py(shell)
+        ret = geos_linearring_from_py(shell)
+        if ret is None:
+            return None
+
+        geos_shell, ndim = ret
         if holes is not None and len(holes) > 0:
             ob = holes
             L = len(ob)
@@ -511,7 +551,7 @@ def geos_polygon_from_py(shell, holes=None):
                 N = exemplar._ndim
             if not L >= 1:
                 raise ValueError("number of holes must be non zero")
-            if not N in (2, 3):
+            if N not in (2, 3):
                 raise ValueError("insufficiant coordinate dimension")
 
             # Array of pointers to ring geometries
@@ -524,19 +564,7 @@ def geos_polygon_from_py(shell, holes=None):
         else:
             geos_holes = POINTER(c_void_p)()
             L = 0
+
         return (
             lgeos.GEOSGeom_createPolygon(
-                        c_void_p(geos_shell),
-                        geos_holes,
-                        L
-                        ),
-            ndim
-            )
-
-# Test runner
-def _test():
-    import doctest
-    doctest.testmod()
-
-if __name__ == "__main__":
-    _test()
+                c_void_p(geos_shell), geos_holes, L), ndim)
