@@ -13,7 +13,6 @@ class MinimumRotatedRectangleTestCase(unittest.TestCase):
         ls = geometry.LineString([(0,1), (1, 2), (2, 1), (1, 0)])
         rect = ls.minimum_rotated_rectangle
         self.assertIsInstance(rect, geometry.Polygon)
-        self.assertIsInstance(rect, geometry.Polygon)
         self.assertEqual(rect.area - ls.convex_hull.area < 0.1, True)
         self.assertEqual(len(rect.exterior.coords), 5)
 
@@ -28,6 +27,3 @@ class MinimumRotatedRectangleTestCase(unittest.TestCase):
         self.assertEqual(len(rect.coords), 2)
         self.assertEqual(rect.coords[0], (0,0))
         self.assertEqual(rect.coords[1], (2,2))
-
-def test_suite():
-    return unittest.TestLoader().loadTestsFromTestCase(MinimumRotatedRectangleTestCase)
\ No newline at end of file
